/****************************************************************************
**
** Copyright (C) 2022 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of Outrun demo.
**
** $QT_BEGIN_LICENSE:GPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 or (at your option) any later version
** approved by the KDE Free Qt Foundation. The licenses are as published by
** the Free Software Foundation and appearing in the file LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "receiver.h"

#include <QtNetwork>

QString Receiver::startServer()
{
    m_incoming.setVersion(QDataStream::Qt_6_0);

    if (m_tcpServer)
        return QString();

    m_tcpServer = new QTcpServer(this);
    if (!m_tcpServer->listen(QHostAddress::Any, m_serverPort)) {
        QString error = tr("Can't start the remote connection: '%1'")
                .arg(m_tcpServer->errorString());
        delete m_tcpServer;
        m_tcpServer = 0;
        return error;
    }

    QList<QHostAddress> ipAddressesList = QNetworkInterface::allAddresses();
    // Use the first non-localhost IPv4 address
    for (int i = 0; i < ipAddressesList.size(); ++i) {
        if (ipAddressesList.at(i) != QHostAddress::LocalHost
                && ipAddressesList.at(i).toIPv4Address()) {
            m_hostAddress = ipAddressesList.at(i);
            emit ipAddressChanged();
            break;
        }
    }

    // If we did not find one, use IPv4 localhost
    if (m_hostAddress.isNull()) {
        m_hostAddress = QHostAddress(QHostAddress::LocalHost);
        emit ipAddressChanged();
    }

    m_serverPort = m_tcpServer->serverPort();
    connect(m_tcpServer, SIGNAL(newConnection()), this, SLOT(acceptRemoteConnection()));
    return QString();
}

void Receiver::disconnectRemote()
{
    if (m_connection)
        m_connection->disconnectFromHost();
}

void Receiver::acceptRemoteConnection()
{
    Q_ASSERT(m_tcpServer);
    Q_ASSERT(!m_connection);
    m_connection = m_tcpServer->nextPendingConnection();
    Q_ASSERT(m_connection);

    m_incoming.setDevice(m_connection);

    connect(m_connection, &QTcpSocket::disconnected,
        this, &Receiver::acceptRemoteDisconnection);

    connect(m_connection, &QTcpSocket::readyRead,
        this, &Receiver::readPayload);

    emit(remoteConnected());
}

void Receiver::acceptRemoteDisconnection()
{
    Q_ASSERT(m_tcpServer);
    Q_ASSERT(m_connection);
    m_connection->deleteLater();
    m_connection = 0;

    m_incoming.setDevice(0);

    emit(remoteDisconnected());
}

void Receiver::readPayload()
{
    m_incoming.startTransaction();

    int totalBytes = 0;
    m_incoming >> totalBytes;

    if (m_connection->bytesAvailable() < totalBytes) {
        m_incoming.rollbackTransaction();
        return;
    }

    QString payloadType;
    m_incoming >> payloadType;
    if (!payloadType.compare(QLatin1String("COMMANDS"))) {
        QString commands;
        m_incoming >> commands;
        m_commands = commands.split(" ");
        emit commandsReceived();
    } else if (!payloadType.compare(QLatin1String("TESTSCRIPT"))) {
        m_incoming >> m_testScript;
        emit testScriptReceived();
    } else {
        qFatal("Unknown payload");
    }
}

